"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseThenRestateHeight = exports.parseHeightInCm = exports.parseHeight = exports.guessHeightUnitFromLocale = exports.parsedHeightToString = exports.inchesToString = exports.parseInches = void 0;

var _intl = require("./intl");

var _metricConversion = require("./metric-conversion");

// @ts-check
const ONLY_NUMBERS_RE = /^\d+$/;
const ONLY_FEET_RE = /^\s*(\d{1,2})[\s_-]*[f'’]\D*$/i;
const ONLY_INCHES_RE = /^\s*(\d{1,3})\D*$/;
const FEET_INCHES_RE = /^([1-9])\D*(0|1|2|3|4|5|6|7|8|9|10|11)\D*$/;
const HAS_METER_UNIT = /\d+[\s_-]*m/i;
const HAS_CM_UNIT = /\d+[\s_-]*c/i;
const SINGLE_DIGIT_WITH_DECIMAL_RE = /^\d?\.\d\d*$/;
const MAX_INCHES = 120; // 10 feet (http://www.guinnessworldrecords.com/world-records/tallest-man-ever)

const MAX_CM = (0, _metricConversion.inchesToCentimeters)(MAX_INCHES);
/**
 * Parses a string that we have decided is an "imperial" height string. and
 * return a number of inches or null if we fail.
 *
 * @param {string} string
 * @returns {number | null}
 */

const parseInches = string => {
  if (ONLY_FEET_RE.test(string)) {
    return parseInt(string, 10) * 12;
  }

  const onlyInches = ONLY_INCHES_RE.exec(string);
  const multiple = FEET_INCHES_RE.exec(string);

  if (onlyInches) {
    return parseInt(onlyInches[1], 10);
  }

  if (multiple) {
    const feet = parseInt(multiple[1], 10);
    const inches = parseInt(multiple[2], 10);
    return feet * 12 + inches;
  }

  return null;
};
/**
 * Take a number of inches and return a string representation of it in feet and inches.
 *
 * @param {number} inches
 * @returns {string}
 */


exports.parseInches = parseInches;

const inchesToString = inches => {
  const feet = Math.floor(inches / 12);
  const inch = inches - feet * 12;
  return "".concat(feet, "'").concat(inch, "\"");
};
/**
 * @param {{ value: number; unit: HeightUnit } | null} heightObj
 * @returns {string}
 */


exports.inchesToString = inchesToString;

const parsedHeightToString = heightObj => {
  if (!heightObj) {
    return '';
  }

  const {
    unit,
    value
  } = heightObj;

  if (unit === 'cm') {
    return "".concat(value, " cm");
  }

  return inchesToString(value);
};
/**
 * Guess the unit based on the browser locale if we have one and we're in a
 * browser context.
 *
 * @returns {HeightUnit}
 */


exports.parsedHeightToString = parsedHeightToString;

const guessHeightUnitFromLocale = () => (0, _intl.getBrowserLocale)() === 'en-US' ? 'in' : 'cm';
/**
 * @param {string} str
 * @param {HeightUnit} [preferredUnit]
 * @returns {{
 *   value: number
 *   unit: HeightUnit
 * } | null}
 */


exports.guessHeightUnitFromLocale = guessHeightUnitFromLocale;

const parseHeightRaw = (str, preferredUnit) => {
  str = str && str.trim ? str.trim() : '';

  if (!str) {
    return null;
  } // handle


  if (ONLY_NUMBERS_RE.test(str)) {
    const value = parseInt(str, 10);
    let unit = preferredUnit || guessHeightUnitFromLocale(); // if there are 3 or more just numbers, we know it
    // has to be cm

    if (str.length > 2) {
      unit = 'cm';
    }

    return {
      value,
      unit
    };
  } // if we have explicit M or a number that can only be m


  if (HAS_METER_UNIT.test(str) || SINGLE_DIGIT_WITH_DECIMAL_RE.test(str)) {
    return {
      value: Math.round(parseFloat(str) * 100),
      unit: 'cm'
    };
  } // if we have explicit CM must be cm.


  if (HAS_CM_UNIT.test(str)) {
    return {
      value: parseInt(str, 10),
      unit: 'cm'
    };
  }

  const inches = parseInches(str);

  if (inches) {
    return {
      value: inches,
      unit: 'in'
    };
  }

  return null;
};
/**
 * Parse height but also sanity check max/min values
 *
 * @param {string} str
 * @param {HeightUnit} [preferredUnit]
 * @returns {{
 *   value: number
 *   unit: HeightUnit
 * } | null}
 */


const parseHeight = (str, preferredUnit) => {
  const result = parseHeightRaw(str, preferredUnit);

  if (!result || result.value <= 0 || isNaN(result.value)) {
    return null;
  }

  const {
    value,
    unit
  } = result;

  if (unit === 'in' && value > MAX_INCHES) {
    return null;
  }

  if (unit === 'cm' && value > MAX_CM) {
    return null;
  }

  return result;
};
/**
 * @param {string} str
 * @param {HeightUnit} [preferredUnit]
 * @returns {number}
 */


exports.parseHeight = parseHeight;

const parseHeightInCm = (str, preferredUnit) => {
  const parsed = parseHeight(str, preferredUnit);

  if (!parsed) {
    return null;
  }

  if (parsed.unit === 'cm') {
    return parsed.value;
  }

  return Math.round((0, _metricConversion.inchesToCentimeters)(parsed.value));
};
/**
 * @param {string} str
 * @param {HeightUnit} [preferredUnit]
 * @returns {string}
 */


exports.parseHeightInCm = parseHeightInCm;

const parseThenRestateHeight = (str, preferredUnit) => parsedHeightToString(parseHeight(str, preferredUnit));

exports.parseThenRestateHeight = parseThenRestateHeight;