"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseThenRestateWeight = exports.parseWeightInKg = exports.parseWeight = exports.parsedWeightToString = exports.parseWeightRaw = exports.guessWeightUnitFromLocale = void 0;

var _metricConversion = require("./metric-conversion");

var _intl = require("./intl");

// @ts-check
const HAS_K_UNIT = /\d+[\s_-]*k/i;
const HAS_LBS_UNIT = /\d+[\s_-]*l/i;
const MAX_KG = 650; // https://en.wikipedia.org/wiki/List_of_the_heaviest_people

const MAX_LBS = (0, _metricConversion.kgToLbs)(MAX_KG);
/**
 * Guess the unit based on the locale
 *
 * @returns {WeightUnit}
 */

const guessWeightUnitFromLocale = () => (0, _intl.getBrowserLocale)() === 'en-US' ? 'lb' : 'kg';
/**
 * @param {string} str
 * @param {WeightUnit | null} preferredUnit
 * @returns {{ value: number; unit: WeightUnit } | null}
 */


exports.guessWeightUnitFromLocale = guessWeightUnitFromLocale;

const parseWeightRaw = (str, preferredUnit) => {
  const value = parseInt(str, 10);

  if (isNaN(value) || value === 0) {
    return null;
  }

  let unit = preferredUnit || guessWeightUnitFromLocale(); // allow overrides in either direction
  // based on actually including the unit

  if (HAS_K_UNIT.test(str)) {
    unit = 'kg';
  }

  if (HAS_LBS_UNIT.test(str)) {
    unit = 'lb';
  }

  return {
    value,
    unit
  };
};
/**
 * @param {{ value: number; unit: WeightUnit } | null} weightObj
 * @returns {string}
 */


exports.parseWeightRaw = parseWeightRaw;

const parsedWeightToString = weightObj => {
  if (!weightObj) {
    return '';
  }

  const {
    unit,
    value
  } = weightObj;
  return "".concat(value, " ").concat(unit);
};
/**
 * Parse weight but also sanity check max/min values
 *
 * @param {string} str
 * @param {WeightUnit | null} preferredUnit
 * @returns {{ value: number; unit: WeightUnit } | null}
 */


exports.parsedWeightToString = parsedWeightToString;

const parseWeight = (str, preferredUnit) => {
  const result = parseWeightRaw(str, preferredUnit);

  if (!result || result.value <= 0) {
    return null;
  }

  const {
    value,
    unit
  } = result;

  if (unit === 'kg' && value > MAX_KG) {
    return null;
  }

  if (unit === 'lb' && value > MAX_LBS) {
    return null;
  }

  return result;
};
/**
 * Parse weight but always return kg
 *
 * @param {string} str
 * @param {WeightUnit} [preferredUnit]
 * @returns {number | null}
 */


exports.parseWeight = parseWeight;

const parseWeightInKg = (str, preferredUnit) => {
  const parsed = parseWeight(str, preferredUnit);

  if (!parsed) {
    return null;
  }

  if (parsed.unit === 'kg') {
    return parsed.value;
  }

  return Math.round((0, _metricConversion.lbsToKg)(parsed.value));
};
/**
 * Parse weight but also sanity check max/min values
 *
 * @param {string} str
 * @param {WeightUnit} [preferredUnit]
 * @returns {string}
 */


exports.parseWeightInKg = parseWeightInKg;

const parseThenRestateWeight = (str, preferredUnit) => parsedWeightToString(parseWeight(str, preferredUnit));

exports.parseThenRestateWeight = parseThenRestateWeight;