"use strict";

var _tape = _interopRequireDefault(require("tape"));

var _throttle = require("./throttle");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const pause = duration => new Promise(resolve => setTimeout(resolve, duration));

const runRepeatedly = (fn, {
  interval,
  duration,
  finalWait
}) => {
  const start = Date.now(); // Call immediately since setInterval waits

  fn();
  return new Promise(resolve => {
    const intId = setInterval(async () => {
      if (Date.now() - start > duration) {
        clearInterval(intId); // To let the final calls all settle

        await pause(finalWait);
        resolve();
      } else {
        fn();
      }
    }, interval);
  });
};

const assertExpectedCalls = (t, calls, expected) => {
  t.deepEqual(Object.keys(calls), Object.keys(expected), 'Has expected number of calls with correct arguments');

  for (const [callNumber, expectedTime] of Object.entries(expected)) {
    const callTime = calls[callNumber];
    t.ok(callTime >= expectedTime - 1 && callTime <= expectedTime + 10, "Call ".concat(callNumber, " is within expected time. Expected: ").concat(expectedTime, " -> ").concat(expectedTime + 10, ", actual: ").concat(callTime));
  }
};

(0, _tape.default)('throttler with leading', async t => {
  let results = {};
  let callNumber = 0;
  let start = Date.now();
  const throttled = (0, _throttle.throttle)(callNumber => {
    results[callNumber] = Date.now() - start;
  }, 100, true); // will try the function at
  // 0, 30, 60, 90, 120, 150
  // Timeline:  0        100       200
  //            +---------+---------+-----
  // Calls:     0  1  2  3  4  5
  // Invoked:   0>>>>>>>>>3>>>>>>>>>5

  await runRepeatedly(() => throttled(callNumber++), {
    interval: 30,
    duration: 160,
    finalWait: 200
  });
  assertExpectedCalls(t, results, {
    // Initial call is immediate
    0: 0,
    // Second execution happens 100ms later, gets args from 3rd call, the most recent before then
    3: 100,
    // Trailing edge call happens another 100ms later, with args from 6th call
    5: 200
  }); // Doing it all again with a break longer than the wait time, and it should all work as expected

  await pause(500); // reset results

  results = {};
  start = Date.now();
  await runRepeatedly(() => throttled(callNumber++), {
    interval: 30,
    duration: 160,
    finalWait: 200
  });
  assertExpectedCalls(t, results, {
    // Initial call is immediate
    6: 0,
    // Second execution happens 100ms later, gets args from 3rd call, the most recent before then
    9: 100,
    // Trailing edge call happens another 100ms later, with args from 6th call
    11: 200
  });
});
(0, _tape.default)('throttle without leading', async t => {
  let results = {};
  let callNumber = 0;
  let start = Date.now();
  const throttled = (0, _throttle.throttle)(callNumber => {
    results[callNumber] = Date.now() - start;
  }, 100, false); // will try the function at
  // 0, 30, 60, 90, 120, 150
  // Timeline:  0        100       200
  //            +---------+---------+-----
  // Calls:     0  1  2  3  4  5
  // Invoked:    >>>>>>>>>3  >>>>>>>>>5
  //
  // NB: without leading, the 5 call here is subtly different to with leading.
  // It is actually called 100ms after the 4 call, instead of 100ms after the 3 invocation.
  // This is how both lodash and underscore seem to be behave and is correct enough vs
  // the complexity avoiding it would add

  await runRepeatedly(() => throttled(callNumber++), {
    interval: 30,
    duration: 160,
    finalWait: 200
  });
  assertExpectedCalls(t, results, {
    // Initial call is skipped, so no 0 entry
    // Second execution happens 100ms later, gets args from 3rd call, the most recent before then
    3: 100,
    // Trailing edge call happens 100ms after the 4 call at 120ms, with args from 6th call
    5: 220
  }); // Doing it all again with a delay longer than the wait time, and it should all work as expected

  await pause(500); // reset results

  results = {};
  start = Date.now();
  await runRepeatedly(() => throttled(callNumber++), {
    interval: 30,
    duration: 160,
    finalWait: 200
  });
  assertExpectedCalls(t, results, {
    // Initial call is skipped again
    // Second execution happens 100ms later, gets args from 3rd call, the most recent before then
    9: 100,
    // Trailing edge call happens 100ms after the 4 call at 120ms, with args from 6th call
    11: 220
  });
});